#! /bin/bash -ex

function undeploy_all {
    cd ${MYPATH}/../deploy
    make undeploy
}

MYPATH="$( cd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1 ; pwd -P )"
source ${MYPATH}/utils.sh
trap undeploy_all EXIT

export TESTOPTS=-v

echo Test deployment of controller and Varnish-as-Ingress
cd ${MYPATH}/../deploy/
make deploy verify

echo "Hello, world!" example
cd ${MYPATH}/../examples/hello/
make deploy verify undeploy

echo "Hello, world!" example with TLS offload
cd ${MYPATH}/../examples/tls/hello
make deploy verify undeploy

echo TLS offload example with multiple certificates distinguished by SNI
cd ${MYPATH}/../examples/tls/sni
make deploy verify undeploy

echo Single namespace example
cd ${MYPATH}/../examples/namespace/
make deploy verify undeploy

echo Varnish Pod template with CLI args example
cd ${MYPATH}/../examples/varnish_pod_template/
make EXAMPLE=cli-args deploy verify undeploy

echo Varnish Pod template with PROXY protocol example
make EXAMPLE=proxy deploy verify undeploy

echo Varnish Pod template with env settings example
make EXAMPLE=env deploy verify undeploy

echo Cluster-wide Ingress example
cd ${MYPATH}/../examples/architectures/clusterwide/
make deploy verify undeploy

echo Example with a cluster-wide Ingress and a namespace-specific Ingress
cd ${MYPATH}/../examples/architectures/cluster-and-ns-wide/
make deploy verify undeploy

echo Multiple Varnish Services in a namespace example
cd ${MYPATH}/../examples/architectures/multi-varnish-ns/
make deploy verify undeploy

echo Multiple Ingress controllers example
cd ${MYPATH}/../examples/architectures/multi-controller/
make deploy verify undeploy

echo Custom VCL example
cd ${MYPATH}/../examples/custom-vcl/
make deploy verify undeploy

echo Self-sharding cluster example
cd ${MYPATH}/../examples/self-sharding/
make deploy verify undeploy

echo Basic Authentication example
cd ${MYPATH}/../examples/authentication/
make EXAMPLE=basic-auth deploy verify undeploy

echo Proxy Authentication example
make EXAMPLE=proxy-auth deploy verify undeploy

echo Combined ACL and Basic Authentication example
make EXAMPLE=acl-or-auth deploy verify undeploy

echo Access control list examples
cd ${MYPATH}/../examples/acl/
make deploy verify undeploy

echo Rewrite rule examples
cd ${MYPATH}/../examples/rewrite/
make deploy verify undeploy

echo Request disposition examples: re-implementing default vcl_recv
cd ${MYPATH}/../examples/req-disposition/
make EXAMPLE=builtin deploy verify undeploy

echo Request disposition examples: alternative re-implemention of default vcl_recv
make EXAMPLE=alt-builtin deploy verify undeploy

echo Request disposition examples: pass on certain cookies, lookup on all others
make EXAMPLE=cookie-pass deploy verify undeploy

echo Request disposition examples: cacheability rules by URL path pattern
make EXAMPLE=cacaheability deploy verify undeploy

echo Request disposition examples: URL white- and blacklisting
make EXAMPLE=url-whitelist deploy verify undeploy

echo Request disposition examples: PURGE method
make EXAMPLE=purge deploy verify undeploy

echo BackendConfig example
cd ${MYPATH}/../examples/backend-config/
./deploy.sh
./verify.sh
./undeploy.sh

echo Ignore deletion of TLS Secrets for non-viking Ingresses
cd ${MYPATH}/e2e/deleteTLSsecret
./deploy.sh
./verify.sh
./undeploy.sh

echo Example of an ExternalName Service as an Ingress backend
cd ${MYPATH}/../examples/externalname/
./deploy.sh
./verify.sh
./undeploy.sh

# Wait for prior versions of the IngressBackends from the previous
# test to delete.
set +e
wait_until_deleted app=coffee,example=externalname
wait_until_deleted app=tea,example=externalname
set -e

# Now re-deploy and verify again -- the Service is assigned a new IP
# address, verify that the new address is resolved.
# Since the Service does not exist for a brief time, DNS lookups get
# negative results during this time. We allow verification failures
# for up to 2 minutes.
./deploy.sh
N=0
while true; do
    if ! ./verify.sh; then
        if [ $N -gt 120 ]; then
            exit 1
        fi
        sleep 10
        N=$(( N + 10 ))
        continue
    else
        break
    fi
done
./undeploy.sh

exit 0
